@@ -20,8 +20,9 @@ from .exceptions import (DisabledNamespace, ExecutionStop, ExistedNamespace,
                          UnexistedNamespace)
 from .interfaces.decorater import DecoraterInterface
 from .interfaces.dispatcher import DispatcherInterface
-from .utilles import (argument_signature, dispatcher_mixin_handler, group_dict, isasyncgen, isgenerator, run_always_await)
+from .utilles import (argument_signature, dispatcher_mixin_handler, group_dict, isasyncgen, isgenerator, run_always_await, run_always_await_safely)
 from .zone import Zone
+from .utilles import printer
 
 class Broadcast:
   loop: asyncio.AbstractEventLoop
@@ -47,6 +48,11 @@ class Broadcast:
     self.listeners = []
     self.dispatcher_inject_rules = inject_rules or []
     self.dispatcher_interface = DispatcherInterface(self)
+
+    @self.dispatcher_interface.inject_global_dispatcher
+    def _(interface: DispatcherInterface):
+      if interface.annotation is interface.event.__class__:
+        return interface.event
   
   def default_listener_generator(self, event_class) -> Listener:
     yield from (iw(self.listeners)
@@ -69,10 +75,7 @@ class Broadcast:
     event: BaseEvent
   ):
     grouped: Dict[int, List[Listener]] = group_dict(listener_generator, lambda x: x.priority)
-    break_flag = False
     for current_priority in sorted(grouped.keys()):
-      if break_flag:
-        break
       current_group = grouped[current_priority]
       try:
         await asyncio.gather(*[
@@ -94,7 +97,8 @@ class Broadcast:
       BaseDispatcher,
     ]] = None,
     hasReferrer: bool = False,
-    enableInternalAccess: bool = False
+    enableInternalAccess: bool = False,
+    use_inline_generator: bool = False
   ):
     is_listener = isinstance(target, Listener)
 
@@ -116,9 +120,11 @@ class Broadcast:
     target_callable = target.callable if is_listener else target
     parameter_compile_result = {}
 
-    async with self.dispatcher_interface.enter_context(event, _dispatchers) as dii:
-      if (not dii.dispatchers) or type(dii.dispatchers[0]) is not DecoraterInterface:
-        DecoraterInterface(dii)  # pylint: disable=unused-variable
+    async with self.dispatcher_interface.enter_context(event, _dispatchers, use_inline_generator) as dii:
+      cached_dispatchers = list(dii.dispatchers)
+      if (not cached_dispatchers) or not isinstance(cached_dispatchers[0], DecoraterInterface): # 理论上只会注入一次.
+        dei = DecoraterInterface(dii)  # pylint: disable=unused-variable
+        self.dispatcher_interface.execution_contexts[0].dispatchers.insert(0, dei)
       # Decorater 的 Dispatcher 已经注入, 没他事了
 
       if enableInternalAccess or \
@@ -138,12 +144,7 @@ class Broadcast:
 
       for injection_rule in self.dispatcher_inject_rules:
         if injection_rule.check(event, dii):
-          dii.dispatchers.insert(1, injection_rule.target_dispatcher)
-
-      @dii.inject_global_dispatcher
-      def _(interface: DispatcherInterface):
-        if interface.annotation is event.__class__:
-          return event
+          dii.execution_contexts[-1].dispatchers.insert(1, injection_rule.target_dispatcher)
 
       try:
         for name, annotation, default in argument_signature(target_callable):
@@ -168,9 +169,9 @@ class Broadcast:
         raise
 
       try:
-        result = await run_always_await(
-          target_callable(**parameter_compile_result)
-        )
+        result = await asyncio.ensure_future(run_always_await_safely(
+          target_callable, **parameter_compile_result
+        ))
       except ExecutionStop:
         raise # 直接抛出.
       except PropagationCancelled:
@@ -184,17 +185,20 @@ class Broadcast:
           ))
         raise
       
+      gener = None
       if [inspect.isgenerator, isgenerator][isinstance(result, Hashable)](result):
-        try: result = next(result)
+        gener = result
+        try: result = next(gener)
         except StopIteration as e: result = e.value
         else: dii.alive_generater_dispatcher[-1].append(
-          (result, False)
+          (gener, False)
         )
       elif [inspect.isasyncgen, isasyncgen][isinstance(result, Hashable)](result):
-        try: result = await result.__anext__()
+        gener = result
+        try: result = await gener.__anext__()
         except StopAsyncIteration: return # value = None
         else: dii.alive_generater_dispatcher[-1].append(
-          (result, True)
+          (gener, True)
         )
 
       if result.__class__ is Force:
