@@ -1,12 +1,14 @@
 import inspect
 from typing import Any, Dict
 
+from graia.broadcast.entities.dispatcher import BaseDispatcher
+
 from ..entities.decorater import Decorater
 from ..entities.signatures import Force
-from ..utilles import is_asyncgener, iscoroutinefunction, isgeneratorfunction, run_always_await
+from ..utilles import is_asyncgener, iscoroutinefunction, isgeneratorfunction, run_always_await_safely
 from .dispatcher import DispatcherInterface
 
-class DecoraterInterface:
+class DecoraterInterface(BaseDispatcher):
     """Graia Broadcast Control 内部机制 Decorate 的具体管理实现
     """
     dispatcher_interface: DispatcherInterface
@@ -16,7 +18,6 @@ class DecoraterInterface:
 
     def __init__(self, dispatcher_interface: DispatcherInterface):
         self.dispatcher_interface = dispatcher_interface
-        self.dispatcher_interface.dispatchers.insert(0, self.decorater_dispatcher)
 
     @property
     def name(self):
@@ -30,7 +31,7 @@ class DecoraterInterface:
     def event(self):
         return self.dispatcher_interface.event
 
-    async def decorater_dispatcher(self, interface: DispatcherInterface):
+    async def catch(self, interface: DispatcherInterface):
         if isinstance(interface.default, Decorater):
             decorater: Decorater = interface.default
             if not decorater.pre:
@@ -48,7 +49,7 @@ class DecoraterInterface:
                     for i in decorater.target(self):
                         yield i
                 else:
-                    yield Force(await run_always_await(decorater.target(self)))
+                    yield Force(await run_always_await_safely(decorater.target, self))
             finally:
                 if not decorater.pre:
                     self.return_value = None
\ No newline at end of file
