@@ -1,11 +1,12 @@
-from typing import Any, AsyncGenerator, Callable, Generator, List, Optional, Tuple, Type, Union, _GenericAlias
+import itertools
+from typing import Any, AsyncGenerator, Callable, Generator, List, Optional, Set, Tuple, Type, Union, _GenericAlias
 
 from ..entities.dispatcher import BaseDispatcher
 from ..entities.event import BaseEvent
 from ..entities.signatures import Force
 from ..exceptions import (OutOfMaxGenerater,
                           RequirementCrashed)
-from ..utilles import is_asyncgener, isgeneratorfunction, run_always_await
+from ..utilles import as_sliceable, is_asyncgener, isgeneratorfunction, run_always_await, run_always_await_safely
 
 def get_raw_dispatcher_callable(dispatcher: Any):
   if isinstance(dispatcher, BaseDispatcher):
@@ -13,51 +14,58 @@ def get_raw_dispatcher_callable(dispatcher: Any):
   if callable(dispatcher):
     return dispatcher
 
-class ContextStackItem:
-  __slots__ = ("dispatchers", "event", "_index")
+class EmptyEvent(BaseEvent):
+  class Dispatcher:
+    @staticmethod
+    def catch(_):
+      pass
+
+class ExecutionContext:
+  __slots__ = ("dispatchers", "always_dispatchers", "event", "inline_generator", "_index")
 
   dispatchers: List[BaseDispatcher]
+  always_dispatchers: Set[Union[BaseDispatcher, Callable]]
   event: BaseEvent
+  inline_generator: bool
   _index: int
 
-  def __init__(self, dispatchers: List[BaseDispatcher], event: BaseEvent) -> None:
+  def __init__(self, dispatchers: List[BaseDispatcher], event: BaseEvent, inline_generator: bool = False) -> None:
     self.dispatchers = dispatchers
     self.event = event
+    self.inline_generator = inline_generator
     self._index = 0
-
-class ExecuteContextStackItem:
-  def __init__(self, name, annotation, default, local_dispatchers, optional=False) -> None:
-    self.name, self.annotation, self.default, self.local_dispatchers = \
-      name, annotation, default, local_dispatchers
-    self.optional = optional
-    self.always_dispatchers = []
-    for i in local_dispatchers:
+  
+    self.always_dispatchers = set()
+    for i in dispatchers:
       if isinstance(i, BaseDispatcher) and i.always:
-        self.always_dispatchers.append(i)
+        self.always_dispatchers.add(i)
+
+class ParameterContext:
+  def __init__(self, name, annotation, default, dispatchers, optional=False) -> None:
+    self.name, self.annotation, self.default, self.dispatchers = \
+      name, annotation, default, dispatchers
+    self.optional = optional
   
   def __repr__(self) -> str:
-    return "<ContextStackItem name={0} annotation={1} default={2} locald={3}"\
+    return "<ParameterContext name={0} annotation={1} default={2} locald={3}"\
       .format(
-        self.name, self.annotation, self.default, self.local_dispatchers)
+        self.name, self.annotation, self.default, self.dispatchers)
 
   name: str
   annotation: Any
   default: Any
-  local_dispatchers: List[Union[BaseDispatcher, Callable]]
+  dispatchers: List[Union[BaseDispatcher, Callable]]
   optional: bool
 
-  # 即 无论如何都会被激活至少 1 次的 Dispatchers.
-  always_dispatchers: List[Union[BaseDispatcher, Callable]]
-
-  __slots__ = ("name", "annotation", "default", "local_dispatchers", "optional", "always_dispatchers")
+  __slots__ = ("name", "annotation", "default", "dispatchers", "optional")
 
 class DispatcherInterface:
-  __slots__ = ("broadcast", "execute_context_stack", "context_stack", "alive_generater_dispatcher")
+  __slots__ = ("broadcast", "execution_contexts", "parameter_contexts", "alive_generater_dispatcher")
 
   broadcast: "Broadcast"
 
-  execute_context_stack: List[ExecuteContextStackItem]
-  context_stack: List[ContextStackItem]
+  execution_contexts: List[ExecutionContext]
+  parameter_contexts: List[ParameterContext]
   alive_generater_dispatcher: List[
     List[Tuple[Union[Generator, AsyncGenerator], bool]]
     # True => async, False => sync
@@ -65,88 +73,106 @@ class DispatcherInterface:
 
   @property
   def name(self):
-    return self.execute_context_stack[-1].name
+    return self.parameter_contexts[-1].name
 
   @property
   def annotation(self):
-    return self.execute_context_stack[-1].annotation
+    return self.parameter_contexts[-1].annotation
 
   @property
   def default(self):
-    return self.execute_context_stack[-1].default
+    return self.parameter_contexts[-1].default
 
   @property
   def _index(self):
-    return self.context_stack[-1]._index
-
-  @_index.setter
-  def _(self, new_value):
-    self.context_stack[-1]._index = new_value
+    return self.execution_contexts[-1]._index
 
   @property
-  def local_dispatchers(self):
-    return self.execute_context_stack[-1].local_dispatchers
+  def global_dispatchers(self):
+    return self.execution_contexts[0].dispatchers
 
   @property
   def dispatchers(self):
-    return self.context_stack[-1].dispatchers
-  
+    return as_sliceable(itertools.chain(
+      self.execution_contexts[0].dispatchers,
+      self.parameter_contexts[-1].dispatchers,
+      self.execution_contexts[-1].dispatchers
+    ))
+
   @property
   def event(self):
-    return self.context_stack[-1].event
+    return self.execution_contexts[-1].event
 
   # dispatcher 允许任何值, 包括 Decorater, 只不过 Decorater Dispatcher 的优先级是隐式最高, 所以基本上不可能截获到.
 
   def __init__(self, broadcast_instance: "Broadcast"):
     self.broadcast = broadcast_instance
     self.alive_generater_dispatcher = [[]]
-    self.context_stack = []
-    self.execute_context_stack =\
-      [ExecuteContextStackItem(None, None, None, [])]
+    self.parameter_contexts = [ParameterContext(None, None, None, [])]
+    self.execution_contexts = [ExecutionContext([], EmptyEvent())]
   
-  def enter_context(self, event: BaseEvent, dispatchers: List[BaseDispatcher]):
-    self.context_stack.append(ContextStackItem(dispatchers, event))
+  def enter_context(self, event: BaseEvent, dispatchers: List[BaseDispatcher], use_inline_generator: bool = False):
+    self.execution_contexts.append(ExecutionContext(dispatchers, event, use_inline_generator))
     for i in self.dispatchers[self._index:]:
       if getattr(i, "always", False):
-        self.always_dispatcher.append(i)
+        self.execution_contexts[-1].always_dispatchers.add(i)
     return self
 
   async def __aenter__(self) -> "DispatcherInterface":
+    self.alive_generater_dispatcher.append([])
     return self
 
   async def __aexit__(self, _, exc, tb):
-    if len(self.alive_generater_dispatcher) != 0:
-      await self.alive_dispatcher_killer()
-      self.alive_generater_dispatcher.pop()
+    if self.alive_generater_dispatcher[-1]:
+      if self.execution_contexts[-1].inline_generator:
+        if len(self.alive_generater_dispatcher) > 2: # 防止插入到保护区
+          self.alive_generater_dispatcher[-2].extend(self.alive_generater_dispatcher[-1])
+        else:
+          raise ValueError("cannot cast to inline")
+      else:
+        await self.alive_dispatcher_killer()
+    self.alive_generater_dispatcher.pop()
 
     for i in self.dispatchers:
       after_execute_call = getattr(i, "after_execute", None)
       if callable(after_execute_call):
         try:
-          await run_always_await(after_execute_call())
+          await run_always_await_safely(after_execute_call)
         except:
           pass
 
-    self.context_stack.pop()
+    self.execution_contexts.pop()
 
     if tb is not None:
       raise exc
 
   def inject_local_dispatcher(self, *dispatchers: List[Union[BaseDispatcher, Callable]]):
-    self.local_dispatchers.extend(dispatchers)
-    always_dispatchers = self.execute_context_stack[-1].always_dispatchers
+    for dispatcher in dispatchers[::-1]:
+      self.parameter_contexts[-1].dispatchers.insert(0, dispatcher)
+    always_dispatchers = self.execution_contexts[-1].always_dispatchers
 
     for i in dispatchers:
       if getattr(i, "always", False):
-        always_dispatchers.append(i)
+        always_dispatchers.add(i)
+
+  def inject_execute_dispatcher(self, *dispatchers: List[Union[BaseDispatcher, Callable]]):
+    for dispatcher in dispatchers:
+      self.execution_contexts[-1].dispatchers.insert(0, dispatcher)
+    always_dispatchers = self.execution_contexts[-1].always_dispatchers
+
+    for i in dispatchers:
+      if getattr(i, "always", False):
+        always_dispatchers.add(i)
 
   def inject_global_dispatcher(self, *dispatchers: List[Union[BaseDispatcher, Callable]]):
-    self.dispatchers.extend(dispatchers)
-    always_dispatchers = self.execute_context_stack[-1].always_dispatchers
+    #self.dispatchers.extend(dispatchers)
+    for dispatcher in dispatchers[::-1]:
+      self.global_dispatchers.insert(1, dispatcher)
+    always_dispatchers = self.execution_contexts[-1].always_dispatchers
 
     for i in dispatchers:
       if getattr(i, "always", False):
-        always_dispatchers.append(i)
+        always_dispatchers.add(i)
 
   async def execute_with(self, name: str, annotation, default):
     optional = False
@@ -158,22 +184,21 @@ class DispatcherInterface:
       else:
         raise TypeError("cannot parse this annotation: {0}".format(annotation))
 
-    self.execute_context_stack.append(ExecuteContextStackItem(name, annotation, default, [], optional=optional))
+    self.parameter_contexts.append(
+      ParameterContext(name, annotation, default, [], optional=optional)
+    )
 
-    alive_dispatchers = []
-    self.alive_generater_dispatcher.append(alive_dispatchers)
-    
-    always_dispatcher = self.execute_context_stack[-1].always_dispatchers
+    alive_dispatchers = self.alive_generater_dispatcher[-1]
+    always_dispatcher = self.execution_contexts[-1].always_dispatchers
     
     result = None
     try:
-      for self.context_stack[-1]._index, dispatcher in enumerate(
+      for self.execution_contexts[-1]._index, dispatcher in enumerate(
           self.dispatchers[self._index+1:],
           start=self._index+1):
         
-        if getattr(dispatcher, "always", False):
-          try: always_dispatcher.remove(dispatcher)
-          except: pass
+        if getattr(dispatcher, "always", False) and dispatcher in always_dispatcher: # 你永远不知道需求有多复杂.
+          always_dispatcher.remove(dispatcher)
 
         if isinstance(dispatcher, type) and issubclass(dispatcher, BaseDispatcher):
           local_dispatcher = dispatcher().catch
@@ -207,7 +232,7 @@ class DispatcherInterface:
               (now_dispatcher_generater, False)
             )
         else:
-          result = await run_always_await(local_dispatcher(self))
+          result = await run_always_await_safely(local_dispatcher, self)
 
         if result is None:
           continue
@@ -215,14 +240,14 @@ class DispatcherInterface:
         if result.__class__ is Force:
           result = result.target
 
-        self.context_stack[-1]._index = 0
+        self.execution_contexts[-1]._index = 0
         return result
       else:
         if optional:
-          self.context_stack[-1]._index = 0
+          self.execution_contexts[-1]._index = 0
           return None
         raise RequirementCrashed("the dispatching requirement crashed: ", self.name, self.annotation, self.default)
-    finally: # 在某种程度上, 这算 BCC 内的第二个 "无头充数" 的设计了.
+    finally:
       for always_dispatcher in always_dispatcher:
         if isinstance(always_dispatcher, type) and issubclass(always_dispatcher, BaseDispatcher):
           local_dispatcher = always_dispatcher().catch
@@ -254,9 +279,9 @@ class DispatcherInterface:
           except StopIteration as e:
             pass
         else:
-          await run_always_await(local_dispatcher(self))
+          await run_always_await_safely(local_dispatcher, self)
 
-      self.execute_context_stack.pop()
+      self.parameter_contexts.pop()
 
   async def alive_dispatcher_killer(self):
     for unbound_gen, is_async_gen in self.alive_generater_dispatcher[-1]:
